-- Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity dvi_transmitter is
	port(
		RST_N		: in  std_logic;
		DCLK : in std_logic;
		SCLK : in std_logic;
    
		data0_in	: in  std_logic_vector(9 downto 0);
		data1_in	: in  std_logic_vector(9 downto 0);
		data2_in	: in  std_logic_vector(9 downto 0);
		
		data0p_h	: out std_logic;
		data0p_l	: out std_logic;
		data0n_h	: out std_logic;
		data0n_l	: out std_logic;

		data1p_h	: out std_logic;
		data1p_l	: out std_logic;
		data1n_h	: out std_logic;
		data1n_l	: out std_logic;

		data2p_h	: out std_logic;
		data2p_l	: out std_logic;
		data2n_h	: out std_logic;
		data2n_l	: out std_logic;

		clockp_h	: out std_logic;
		clockp_l	: out std_logic;
		clockn_h	: out std_logic;
		clockn_l	: out std_logic
	);
end dvi_transmitter;

architecture RTL of dvi_transmitter is

	signal data0_latch	: std_logic_vector(9 downto 0);
	signal data1_latch	: std_logic_vector(9 downto 0);
	signal data2_latch	: std_logic_vector(9 downto 0);

	signal data0_ser: std_logic_vector(9 downto 0);
	signal data1_ser: std_logic_vector(9 downto 0);
	signal data2_ser: std_logic_vector(9 downto 0);

	signal clock_ser: std_logic_vector(9 downto 0);

	signal start_sig	: std_logic_vector(4 downto 0);

begin

-------------------------------------------------------------- 
process( RST_N, DCLK )  
begin 
   if( RST_N = '0' ) then
     data0_latch <= (others => '0');
     data1_latch <= (others => '0');
     data2_latch <= (others => '0');
   elsif( DCLK'event and DCLK = '1' ) then
     data0_latch <= data0_in;
     data1_latch <= data1_in;
     data2_latch <= data2_in;
   end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, SCLK )  
begin 
   if( RST_N = '0' ) then
     start_sig <= "00001";
   elsif( SCLK'event and SCLK = '1' ) then
     start_sig <= start_sig(0) & start_sig(4 downto 1);
   end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, SCLK )  
begin 
   if( RST_N = '0' ) then
     data0_ser <= (others => '0');
     data1_ser <= (others => '0');
     data2_ser <= (others => '0');
     clock_ser <= (others => '0');
   elsif( SCLK'event and SCLK = '1' ) then
      if (start_sig(0) = '1') then
         data0_ser <= data0_latch;
         data1_ser <= data1_latch;
         data2_ser <= data2_latch;
         clock_ser <= "0000011111";
      else
         data0_ser <= "00" & data0_ser(9 downto 2);
         data1_ser <= "00" & data1_ser(9 downto 2);
         data2_ser <= "00" & data2_ser(9 downto 2);
         clock_ser <= "00" & clock_ser(9 downto 2);
      end if;
   end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, SCLK )  
begin 
   if( RST_N = '0' ) then
	data0p_h <= '0';
	data0p_l <= '0';
	data0n_h <= '0';
	data0n_l <= '0';

	data1p_h <= '0';
	data1p_l <= '0';
	data1n_h <= '0';
	data1n_l <= '0';

	data2p_h <= '0';
	data2p_l <= '0';
	data2n_h <= '0';
	data2n_l <= '0';

	clockp_h <= '0';
	clockp_l <= '0';
	clockn_h <= '0';
	clockn_l <= '0';
   elsif( SCLK'event and SCLK = '1' ) then
	data0p_h <= data0_ser(0);
	data0p_l <= data0_ser(1);
	data0n_h <= not data0_ser(0);
	data0n_l <= not data0_ser(1);

	data1p_h <= data1_ser(0);
	data1p_l <= data1_ser(1);
	data1n_h <= not data1_ser(0);
	data1n_l <= not data1_ser(1);

	data2p_h <= data2_ser(0);
	data2p_l <= data2_ser(1);
	data2n_h <= not data2_ser(0);
	data2n_l <= not data2_ser(1);

	clockp_h <= clock_ser(0);
	clockp_l <= clock_ser(1);
	clockn_h <= not clock_ser(0);
	clockn_l <= not clock_ser(1);
   end if;  
end process;

end RTL;

