--------------------------------------------------
-- env.vhd
-- by Toshio Iwata at digitalfilter.com 2016/03/29

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.STD_LOGIC_UNSIGNED.ALL;
  use IEEE.std_logic_arith.all;

entity env is
   Port (
          CLK : out std_logic; 
          CLK100 : out std_logic; 
          RST_N : out std_logic;
          PCLK : out std_logic;
    DCLK : out std_logic;
    SCLK : out std_logic;
          CamHsync : out std_logic;
          CamVsync : out std_logic;
          CamData : Out std_logic_vector(7 downto 0) );
end env;

architecture beh of env is

  constant sv_clk_period : time := 20 ns; -- 50MHz 
  constant sv_clk_value : real := 20.0e-9; -- 50MHz

  constant sv_pclk_period : time := 40 ns; -- 25MHz 
  constant sv_pclk_value : real := 40.0e-9; -- 25MHz

  constant sv_ser_period : time := 8 ns; -- 125MHz 
  constant sv_ser_value : real := 8.0e-9; -- 125MHz

  constant sv_chsync_period : time := 62720 ns; -- 80ns * 784 (15.9khz)
  constant sv_chsync_value : real := 62720.0e-9; 

  constant sv_cvsync_period : time := 31987200 ns; -- 80ns * 784 * 510 (31.2hz)
  constant sv_cvsync_value : real := 31987200.0e-9; 

signal yin_sig : std_logic_vector(7 downto 0);
signal lineco	: integer := 0;
signal CamHsync_sig	: std_logic;

begin

process
begin
  CLK <= '0';
  wait for sv_clk_period / 2.0;
  CLK <= '1';
  wait for sv_clk_period - sv_clk_period / 2.0;
end process;

process
variable first_time : boolean := true;
begin
  if(first_time) then
    wait for 1 ns;
    first_time := false;
  end if;
  CLK100 <= '0';
  wait for sv_clk_period / 4.0;
  CLK100 <= '1';
  wait for sv_clk_period / 2.0 - sv_clk_period / 4.0;
end process;

process
begin
  SCLK <= '0';
  wait for sv_ser_period / 2.0;
  SCLK <= '1';
  wait for sv_ser_period - sv_ser_period / 2.0;
end process;

process
begin
  PCLK <= '1';
  DCLK <= '1';
  wait for sv_pclk_period / 2.0;
  PCLK <= '0';
  DCLK <= '0';
  wait for sv_pclk_period - sv_pclk_period / 2.0;
end process;

process
begin
  if(lineco = 509) then
    lineco <= 0;
  else
    lineco <= lineco + 1;
  end if;
  CamHsync_sig <= '0';
  wait for sv_pclk_period * 144;
  CamHsync_sig <= '1';
  wait for sv_chsync_period - sv_pclk_period * 144;
end process;

  CamHsync <= CamHsync_sig when lineco >= 20 and lineco < 500 else '0';

process
begin
  CamVsync <= '1';
  wait for sv_chsync_period * 3;
  CamVsync <= '0';
  wait for sv_cvsync_period - sv_chsync_period * 3;
end process;

process
variable first_time : boolean := true;
variable pi : real := 3.14159;
variable t : real := 0.0;
variable idata : integer := 0;
begin
  if( first_time ) then
    wait for sv_pclk_period/2;
    first_time := false;
    idata := -1;
  end if;
  if(idata = 255) then
    idata := 0;
  else 
    idata := idata + 1;
  end if;
  yin_sig <= conv_std_logic_vector(idata, 8);
  wait for sv_pclk_period;
  t := t + sv_pclk_value;
end process;

  CamData <= yin_sig;

process
begin
   RST_N <= '0';
   wait for 1 us;
   RST_N <= '1';
   wait;
end process ;

end beh;
