`timescale 1ns/1ps

module env (
	RST_N,
	CLK,
	CLK100M,
	PCLK,
	DCLK,
	SCLK,
	CamHsync,
	CamVsync,
	CamData,
 ); 

parameter clk_period = 20; // 20 ns
parameter sdramclk_period = 10; // 10 ns
parameter pclk_period = 40; // 40 ns
parameter sclk_period = 8; // 8 ns
parameter chsync_period = 62720; // 62720 ns
parameter cvsync_period = 31987200; // 31987200 ns

output		CLK, CLK100M, RST_N, PCLK, DCLK, SCLK, CamHsync, CamVsync;
output [7:0] CamData;
reg ClockFast, ClockSdram, ClockHalf, ResetN, ClockDvi;
reg CamHsync_reg;
integer	lineco;
reg CamVsync_reg;
integer idata;

assign CLK = ClockFast;
assign CLK100M= ClockSdram;
assign PCLK = ClockHalf;
assign DCLK = ClockHalf;
assign SCLK = ClockDvi;
assign RST_N = ResetN;
assign CamHsync = (lineco >= 20) ? CamHsync_reg : 0;
assign CamVsync = CamVsync_reg;
assign CamData = idata;

	initial ClockFast = 0;
	always #(clk_period/2) ClockFast = !ClockFast;

	initial ClockSdram = 0;
	always #(sdramclk_period/2) ClockSdram = !ClockSdram;

	initial ClockHalf = 0;
	always #(pclk_period/2) ClockHalf = !ClockHalf;
	
	initial ClockDvi = 0;
	always #(sclk_period/2) ClockDvi = !ClockDvi;
	
	initial CamHsync_reg = 0;
	initial lineco = 0;
	always 
	begin
	  if(lineco == 509) begin
	    lineco = 0;
	  end else begin 
     lineco = lineco+1;
    end
    #(pclk_period * 144) begin 
      CamHsync_reg = 1;
    end
	  #(chsync_period - pclk_period * 144) begin
	    CamHsync_reg = 0;
    end 
	end
	
	initial idata = 0;
	always 
	begin
    #(pclk_period) begin 
	    if(idata == 255) begin
	      idata = 0;
	    end else begin 
       idata = idata+1;
      end
    end
	end
	
	initial CamVsync_reg = 0;
	always 
	begin
    #(chsync_period * 3) CamVsync_reg = 1;
	  #(cvsync_period - chsync_period * 3) CamVsync_reg = 0;
 	end
	
	initial begin
		ResetN = 1'b1;
		#1 ResetN = 1'b0;
		#2 ResetN = 1'b1;
	end

endmodule
